#include "library.h"

const double pi = acos(-1.0);				
const double ninety = pi/2, degrees = pi/180;				//conversions
string readstuff;											//Input data for exiting, mode, drawing another tree
int red;
int green;
int blue;													//Color values are stored as global variables so they can be used across functions
double ypos;												//current position of y

void leaves(double cx, double cy, int counter)
{
for (int i=counter; i>0; i--)
	{
	move_to(cx,cy);													//Center point which leaves are randomly generated around.
	double randang = (double)random_in_range(0,359)/degrees;		//Random angle to draw leaves around the center point.
	set_heading_radians(randang);
	double randvect = (double)random_in_range(1,10);				//Random length that the leaves are away from the center point.
	move_distance(randvect);
	double randpen = (double)random_in_range(1,5);					//Random size of leaves
	set_pen_width(randpen);											
	red=0; blue=0; green=random_in_range(75,230);					//Random color within specific set of shades of green
	if (readstuff=="psychadelic")
	{
		red=random_in_range(0,255);
		blue=random_in_range(0,255);
		green=random_in_range(0,255);								//Psychadelic mode uses completely random color for leaves.
	}
	set_pen_color_int(red,green,blue);
	draw_point();
	}
}
bool tree(double p1x, double p1y, double p2x, double p2y, double len, double beta, double end, int color) // (p1x, p1y), (p2x, p2y) starting points for quadrilateral.  len=how tall the shape is.  beta=deviation from 90 degrees of angle.  end=length of segment opposite p1 and p2.
{
	if(end > .003)													//Branches will stop being drawn if the ending size of the top of the segment is too short
	{
	double base = sqrt((p1x-p2x)*(p1x-p2x) + (p1y-p2y)*(p1y-p2y));			// magnitude of vector between p1 and p2
	 double alpha = (atan2(p2x-p1x, p2y-p1y)-ninety);					// Difference in the angle between the slope of p1 and p2 and vertical
	if(readstuff=="psychadelic")
		{
		red=random_in_range(0,255);
		blue=random_in_range(0,255);
		green=random_in_range(0,255);
		}															//In psychadelic mode branch segments are random color
	else
		{
		red=color;
		green=69;
		blue=19;													//In normal mode the trunk is brown with slightly changing shade of red
		}
	set_pen_color_int(color, 69, 19);
	if (sin(beta) > 2*len/base)
		return false;
	double p3x = (p1x + p2x) * 0.5;
	double p3y = (p1y + p2y) * 0.5;
	double p4x = p3x + len * sin(alpha+beta);
	double p4y = p3y - len * cos(alpha+beta);
	double centerx = p3x + .75*len * sin(alpha+beta);  
	double centery = p3y - .75*len * cos(alpha+beta);		//Chooses a point three quarters  up the way of the block to center leaf drawing around.
 	double dx = 0.5 * end * sin(ninety-alpha-beta);
	double dy = 0.5 * end * cos(ninety-alpha-beta);
	double p5x = p4x - dx;
	double p5y = p4y - dy;
	double p6x = p4x + dx;
	double p6y = p4y + dy;
	move_to(p1x, p1y);
	start_shape();
	draw_to(p2x, p2y);
	note_position();
	draw_to(p6x, p6y);
	note_position();
	draw_to(p5x, p5y);
	note_position();
	draw_to(p1x, p1y);
	note_position();
	set_pen_color_int(red,green,blue);
	fill_shape();											//Shape will be filled based on random points
	double betar=(double)random_in_range(15,45)/100;		//Beta will be offset by a random amount for each recursion
	double lenr=(double)random_in_range(70,85)/100;			//Length is decreased by a random ratio for each recursion.
	double endr=(double)random_in_range(175,250)/100;		//End size is decreased by a random ratio for each recursion.
	if (end<4)												//When the branches become small enough, leaves will be drawn.
		{
		int leafcounter=random_in_range(40,80);				//Random number of leaves generated for each recursion.
		leaves(centerx,centery,leafcounter);
		}		
	ypos=get_y_position();
	if(ypos<10)
		end=0;												//Stop tree growth if approaching top of window
	tree(p5x,p5y,p4x,p4y,len*lenr,beta-betar,end/endr,color+5);
	tree(p4x,p4y,p6x,p6y,len*lenr,beta+betar,end/endr,color+5);	//Each branch has two more branches coming off of it, the left angle will be offset by -beta and the right by +beta.
	}
  return true; 
}
void main()
{ 
	while(readstuff!="exit")
	{
	clear();
	make_window(1000, 1000);
	int c = make_color_int(0,160,200);				//Background default is blue
	if (readstuff=="psychadelic")
		c = make_color_int(0,0,0);					//Background black in psychadelic mode
	fill_rectangle(0,0,1000,1000,c);
	c = make_color_int(100,69,19);					//Ground default brown
	if (readstuff=="psychadelic")
		c = make_color_int(0,255,255);				//Ground cyan in psychadelic mode.
	fill_rectangle(0,980,1000,20,c);
	
	red = 100 + random_in_range(0,40);				// Choosing a center value of red that will return a color close to brown and deviating from it by a standard amount.
	double width = random_in_range(30,100);			//Initial width range
	double beta = random_in_range(-5,5)*degrees;	//Initial beta angle range
	double initlen = random_in_range(175,250);		//Initial length range
    tree (500-width*.5, 980, 500+width*.5 , 980, initlen, beta, initlen*.3, red);
	cout << "Type anything and press enter to draw a new tree.  Type 'exit' to close.  Type 'psychadelic' for a super awesome tree. ";
	cin >> readstuff;
	}
}